﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'  CsvReaderクラス
'
'  Copyright (c) 2016, Advanced Core Technologies, Inc. All Rights Reserved.
'
'  2016/02/05 Rev.1.0.0.0 MOTO VB.NET版作成
'
'//////////////////////////////////////////////////////////////////////////////////////////////////

Imports System.IO
Imports System.Text.RegularExpressions

Namespace Act.VisualBasic.Utilities.CsvIO

    ''' <summary>
    ''' ストリームや文字列からCSVを読み取ります。
    ''' </summary>
    Public Class CsvReader
        Implements IDisposable

        '==========================================================================================
        '  Constructors
        '==========================================================================================
#Region "Constructors"

        ''' <summary>
        ''' 指定したTextReaderと見出し行の有無、区切り文字、引用符を使用して初期化します。
        ''' </summary>
        ''' <param name="reader">読み取りを行うTextReader</param>
        ''' <param name="hasHeaderRecord">見出し行の有無。見出し行がある場合true</param>
        ''' <param name="delimiter">区切り文字</param>
        ''' <param name="quote">引用符</param>
        Public Sub New(ByVal reader As TextReader,
                       Optional ByVal hasHeaderRecord As Boolean = False,
                       Optional ByVal delimiter As Delimiter = Delimiter.Comma,
                       Optional ByVal quote As QuotationMark = CsvIO.QuotationMark.Double)
            Me.reader_ = reader
            Me.HasHeaderRecord = hasHeaderRecord
            Me.Delimiter = delimiter
            Me.QuotationMark = quote
            Me.SkipEmptyLines = True
            Me.TrimQuotes = True
            Me.RemoveEscapeDoubleQuotes = True
        End Sub

#End Region

        '==========================================================================================
        '  Private Constants
        '==========================================================================================
#Region "Private Constants"

        Private Shared ReadOnly DoubleQuote As Char = """"c

        ''' <summary>
        ''' レコードを分割する正規表現フォーマットです。
        ''' </summary>
        Private Shared ReadOnly SplitRecordPatternFormat As String =
            "(""(?:[^""]|"""")*""|[^{0}]*){0}"

#End Region

        '==========================================================================================
        '  Private Fields
        '==========================================================================================
#Region "Private Fields"

        ''' <summary>
        ''' 見出し行の読み込み済みフラグです。
        ''' </summary>
        Private alreadyReadHeader_ As Boolean

        ''' <summary>
        ''' Dispose済みフラグです。
        ''' </summary>
        Private alreadyDisposed_ As Boolean

        ''' <summary>
        ''' Readerのバッキングフィールドです。
        ''' </summary>
        ''' <remarks></remarks>
        Private reader_ As TextReader

#End Region

        '==========================================================================================
        '  Properties
        '==========================================================================================
#Region "Properties"

        ''' <summary>
        ''' 読み取りを行うTextReaderを取得します。
        ''' </summary>
        Public ReadOnly Property Reader As TextReader
            Get
                Return reader_
            End Get
        End Property

        ''' <summary>
        ''' 見出し行が存在するかどうかを示す値を取得または設定します。
        ''' </summary>
        Public Property HasHeaderRecord As Boolean

        ''' <summary>
        ''' 区切り文字を取得または設定します。
        ''' </summary>
        Public Property Delimiter As Delimiter

        ''' <summary>
        ''' 区切り文字を取得します。
        ''' </summary>
        Public ReadOnly Property DelimiterCharacter As Char
            Get
                Dim delimiterChar As Char = vbNullChar
                Select Case Delimiter
                    Case CsvIO.Delimiter.Comma
                        delimiterChar = ","c
                    Case CsvIO.Delimiter.Tab
                        delimiterChar = vbTab
                End Select

                Return delimiterChar
            End Get
        End Property

        ''' <summary>
        ''' 引用符を取得または設定します。
        ''' </summary>
        Public Property QuotationMark As QuotationMark

        ''' <summary>
        ''' 空行をスキップするかどうをかを示す値を取得または設定します。
        ''' </summary>
        ''' <remarks>スキップしない場合、空行は1つの空白フィールドを持つレコードとして読み込まれます。</remarks>
        Public Property SkipEmptyLines As Boolean

        ''' <summary>
        ''' フィールドの先頭と末尾の引用符を削除するかどうかを示す値を取得または設定します。
        ''' </summary>
        Public Property TrimQuotes As Boolean

        ''' <summary>
        ''' エスケープ文字を削除するかどうかを示す値を取得または設定します。
        ''' </summary>
        Public Property RemoveEscapeDoubleQuotes As Boolean

#End Region

        '==========================================================================================
        '  Public Methods
        '==========================================================================================
#Region "Public Methods"

        ''' <summary>
        ''' ストリームまたは文字列から1レコードを読み取り返します。
        ''' </summary>
        ''' <returns>読み取り結果のCsvRecordオブジェクト。最後まで読み取り済みの場合はnull。</returns>
        Public Function ReadRecord() As CsvRecord
            Dim record As String = GetRecord(Reader, QuotationMark)
            If record Is Nothing Then
                Return Nothing
            End If

            ' プロパティの設定に応じて空行をスキップ
            If SkipEmptyLines Then
                If record = String.Empty Then
                    Return ReadRecord()
                End If
            End If
            ' 見出し行が存在し、まだ読み込んでいない場合は次のレコードを返す
            If HasHeaderRecord Then
                If Not alreadyReadHeader_ Then
                    alreadyReadHeader_ = True
                    Return ReadRecord()
                End If
            End If

            Dim tokens As String() = SplitRecord(record, DelimiterCharacter)
            For i As Integer = 0 To tokens.Length - 1
                ' プロパティの設定に応じて、先頭と末尾の引用符を削除
                If TrimQuotes Then
                    If QuotationMark = CsvIO.QuotationMark.Double Then
                        tokens(i) = TrimQuotationMark(tokens(i), DoubleQuote)
                    End If
                End If
                ' プロパティの設定に応じてエスケープ文字を削除
                If RemoveEscapeDoubleQuotes Then
                    tokens(i) = RemoveEscapeDoubleQuote(tokens(i))
                End If
            Next

            Return New CsvRecord(Delimiter, QuotationMark, tokens)
        End Function

        ''' <summary>
        ''' ストリームまたは文字列から全レコードを読み取り返します。
        ''' </summary>
        ''' <returns>読み取り結果のCsvRecordオブジェクト配列。</returns>
        Public Function ReadAllRecords() As CsvRecord()
            Dim records As New List(Of CsvRecord)()
            Dim record As CsvRecord = ReadRecord()
            While record IsNot Nothing
                records.Add(record)
                record = ReadRecord()
            End While

            Return records.ToArray()
        End Function

#End Region

        '==========================================================================================
        '  Private Methods
        '==========================================================================================
#Region "Private Methods"

        ''' <summary>
        ''' 引用符に囲まれた改行は無視しして、CSVの1レコードを返します
        ''' </summary>
        ''' <param name="reader">読み取りを行うTextReader</param>
        ''' <param name="quotationMark">引用符</param>
        ''' <returns>CSVの1レコード分の文字列。</returns>
        Private Shared Function GetRecord(ByVal reader As TextReader,
                                          ByVal quotationMark As QuotationMark) As String
            If reader Is Nothing Then
                Throw New ArgumentNullException("reader")
            End If

            Dim quoteCount As Integer = 0
            Dim concated As String = Nothing
            While (reader.Peek() <> -1)
                Dim line As String = reader.ReadLine()
                ' 文字列を連結する
                concated = If(concated Is Nothing, line, concated & Environment.NewLine & line)

                ' 引用符なしの場合はループを抜ける
                If quotationMark = CsvIO.QuotationMark.None Then
                    Exit While
                End If
                ' 引用符の数が偶数になったらループを抜ける
                quoteCount += CountChar(line, DoubleQuote)
                If (quoteCount Mod 2) = 0 Then
                    Exit While
                End If

            End While

            Return concated
        End Function

        ''' <summary>
        ''' CSVの1レコードを区切り文字で分割して返します。
        ''' </summary>
        ''' <param name="record">>1レコード分の文字列</param>
        ''' <param name="delimiter">区切り文字</param>
        ''' <returns>区切り文字で分割した配列。</returns>
        Private Shared Function SplitRecord(ByVal record As String,
                                            ByVal delimiter As Char) As String()
            If record Is Nothing Then
                Throw New ArgumentNullException("record")
            End If

            ' 指定区切り文字で正規表現オブジェクトを作成
            Dim splitRecordPattern As String = String.Format(SplitRecordPatternFormat, delimiter)
            Dim splitter As New Regex(splitRecordPattern, RegexOptions.Multiline)

            Dim tokens As New List(Of String)()
            For Each match In splitter.Matches(record & delimiter)
                Dim token As String = match.value
                ' 追加した末尾の区切り文字と先頭の空白を削除して追加
                tokens.Add(token.Substring(0, token.Length - 1).Trim())
            Next

            Return tokens.ToArray()
        End Function

        ''' <summary>
        ''' 連続した2個のダブルクォートを1個削除します。
        ''' </summary>
        ''' <param name="value">対象の文字列</param>
        ''' <returns>連続したダブルクォートの１つを削除した文字列。</returns>
        Private Shared Function RemoveEscapeDoubleQuote(ByVal value As String) As String
            If value Is Nothing Then
                Throw New ArgumentNullException("value")
            End If

            Return value.Replace("""""", """")
        End Function

        ''' <summary>
        ''' valueの先頭と末尾から区切り文字を削除します。
        ''' </summary>
        ''' <param name="value">対象の文字列</param>
        ''' <param name="quote">区切り文字</param>
        ''' <returns>削除した文字列。区切り文字がない場合、valueの長さが1以下の場合は元の文字列。</returns>
        Private Shadows Function TrimQuotationMark(ByVal value As String,
                                                   ByVal quote As Char) As String
            If value Is Nothing Then
                Throw New ArgumentNullException("value")
            End If

            If value.Length > 1 AndAlso value(0) = quote AndAlso
                value(value.Length - 1) = quote Then
                Return value.Substring(1, value.Length - 2)
            End If

            Return value
        End Function

        ''' <summary>
        ''' targetの文字列に含まれるvalueの数を返します。
        ''' </summary>
        ''' <param name="target">対象の文字列</param>
        ''' <param name="value">検索する文字</param>
        ''' <returns>valueが含まれている数。targetまたはvalueがnullの時は0。</returns>
        Private Shared Function CountChar(ByVal target As String, ByVal value As Char) As Integer
            If target Is Nothing Then
                Return 0
            End If

            Return target.Length - target.Replace(value.ToString(), String.Empty).Length
        End Function

#End Region

        '==========================================================================================
        '  IDisposable
        '==========================================================================================
#Region "IDisposable"

        Public Sub Dispose() Implements IDisposable.Dispose
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub

        Protected Overridable Sub Dispose(ByVal disposing As Boolean)
            If alreadyDisposed_ Then
                Return
            End If

            If disposing Then
                ' マネージリソースの解放
                Reader.Dispose()
            End If
            ' アンマネージリソースの解放

            alreadyDisposed_ = True
        End Sub

        Protected Overrides Sub Finalize()
            Dispose(False)
        End Sub

#End Region

    End Class


End Namespace

