﻿'//////////////////////////////////////////////////////////////////////////////////////////////////
' 
'  ACT距離計算サービス サンプルプログラム
'  MainFormクラス
'
'  Copyrights (c) 2016-2023, Advanced Core Technologies, Inc. All rights reserved.
'  
'  2023/09/21 Rev.1.0.0.1 KYON AdminURLに合わせてSSL使用フラグを変更出来るように変更
'  2016/02/09 Rev.1.0.0.0 MOTO 作成
'
'//////////////////////////////////////////////////////////////////////////////////////////////////

Imports System.IO
Imports System.Text
Imports System.Text.RegularExpressions
Imports ACT.Dcws
Imports ACTSOAPVBCalcArea.Act.VisualBasic.Utilities.CsvIO

Public Class MainForm

    '==============================================================================================
    '  Private Fields
    '==============================================================================================
#Region "Private Fields"

    ''' <summary>
    ''' ユーザー情報です。
    ''' </summary>
    Private userSettings_ As UserSettings

    ''' <summary>
    ''' 距離計算サービスのユーザーです。
    ''' </summary>
    Private user_ As User

#End Region

    '==============================================================================================
    '  Private Constants
    '==============================================================================================
#Region "Private Constants"

    ''' <summary>
    ''' 経度の列番号です。
    ''' </summary>
    Private Shared ReadOnly LonColumnIndex As Integer = 2

    ''' <summary>
    ''' 緯度の列番号です。
    ''' </summary>
    Private Shared ReadOnly LatColumnIndex As Integer = 3

#End Region

    '==============================================================================================
    '  Private Methods
    '==============================================================================================
#Region "Private Methods"

    ''' <summary>
    ''' 計算を実行し実行結果をファイルに出力します。
    ''' </summary>
    ''' <param name="userSettings">ユーザー設定</param>
    ''' <returns>実行結果のメッセージ。</returns>
    Private Function DoCalc(ByVal userSettings As UserSettings) As String
        Dim message As New StringBuilder()
        Try
            '--------------------------------------------------------------------------------------
            '  管理サービスオブジェクトと環境設定オブジェクトの作成
            '--------------------------------------------------------------------------------------
            ' 管理サービスオブジェクトを作成
            Dim adminService As New DCAdmin.ACTDCADMIN()
            ' エンドポイントURLを設定
            adminService.Url = user_.AdminUrl

            ' 環境設定オブジェクトを作成
            Dim adminEnviron As DCAdmin.Environ = user_.GetAdminEnviron()

            '--------------------------------------------------------------------------------------
            '  ユーザー情報オブジェクトの取得
            '--------------------------------------------------------------------------------------
            ' ユーザー情報オブジェクトを作成
            Dim userInfo As New DCAdmin.UserInformation2()
            ' ユーザー情報取得サービスの呼び出し
            If Not adminService.GetUserInformation2(adminEnviron, userInfo) Then
                ' ユーザー情報の取得に失敗した場合
                message.AppendLine("ユーザー情報の取得に失敗しました。")
                Return message.ToString()
            End If
            user_.UserInfo = userInfo

            '--------------------------------------------------------------------------------------
            '  計算用道路データの取得と選択
            '--------------------------------------------------------------------------------------
            ' 計算用道路データを格納する配列の作成
            Dim nwInfos() As DCAdmin.NWInfo = Nothing
            ' 計算用道路データ取得サービスの呼び出し
            If Not adminService.GetUsableNWInfos(adminEnviron, nwInfos) Then
                ' 計算用道路データの取得に失敗した場合
                message.AppendLine("計算用道路データの取得に失敗しました。")
                Return message.ToString()
            End If

            ' 距離計算に使用する計算用道路データを選択
            Dim nwInfo As DCAdmin.NWInfo = SelectNwInfo(nwInfos, CalcSettings.NWIDPattern)
            If nwInfo Is Nothing Then
                ' 指定の計算用道路データが含まれていない場合
                message.AppendLine("利用可能な計算用道路データに指定のデータが含まれていません。")
                Return message.ToString()
            End If
            message.AppendLine("計算用道路データ: " & nwInfo.Note)

            '--------------------------------------------------------------------------------------
            '  距離計算サービスオブジェクトと距離計算サービス環境設定構造体オブジェクトの作成
            '--------------------------------------------------------------------------------------
            ' 距離計算サーバー取得サービスの呼び出し
            If Not adminService.GetUsableDCServer(adminEnviron, nwInfo) Then
                ' 距離計算サーバーの取得に失敗した場合
                message.AppendLine("距離計算サーバーの情報取得に失敗しました。")
                Return message.ToString()
            End If

            ' 距離計算サービスオブジェクトの作成
            Dim dcService As New DCService.ACTDCWS()
            ' エンドポイントURLを設定
            dcService.Url = Utilities.ChangeProtocol(adminEnviron.WSURL, CalcSettings.UseSsl)
            message.AppendLine("距離計算サーバー: " & dcService.Url & "(" & adminEnviron.Host &
                               ":" & adminEnviron.Port & ")")

            ' 距離計算サービス環境設定構造体オブジェクトを作成
            Dim dcEnviron As DCService.Environ = Utilities.GetDCServiceEnviron(adminEnviron)

            '--------------------------------------------------------------------------------------
            '  CSVファイルを読み込みノードポイントを作成
            '--------------------------------------------------------------------------------------
            ' ファイルから地点の経度緯度情報を読み込む
            Dim records() As CsvRecord = ReadCsvFile(userSettings.InputFilePath)
            If records Is Nothing Then
                ' CSVファイルの読み込みに失敗した場合
                message.AppendLine("入力ファイルの読み込みに失敗しました。")
                Return message.ToString()
            ElseIf records.Length > 1 Then
                ' スタート地点が2地点以上指定されている場合
                message.AppendLine("スタート地点は1地点のみ指定可能です。")
                Return message.ToString()
            End If

            ' スタート地点を設定するノードポイントを作成
            Dim startPnt As New DCService.NodePnt With {.Lon = records(0)(LonColumnIndex),
                                                        .Lat = records(0)(LatColumnIndex)}

            '--------------------------------------------------------------------------------------
            '  到達圏／流入圏計算用パラメータ構造体の設定
            '--------------------------------------------------------------------------------------
            ' 計算範囲を種別にあわせて換算（時間圏=0.1秒単位、距離圏=m単位）
            Dim areaRange As Integer = 0
            Select Case userSettings.AreaKind
                Case AreaKind.Time
                    ' 時間圏の場合
                    areaRange = userSettings.AreaRange * 60 * 10
                Case AreaKind.Dist
                    ' 距離圏の場合
                    areaRange = userSettings.AreaRange * 1000
            End Select

            ' 到達圏／流入圏計算用パラメータ構造体オブジェクトを作成
            Dim param As New DCService.CA3Param With {.Datum = CalcSettings.Datum,
                                                      .CoordinateFormat =
                                                        CalcSettings.CoordinateFormat,
                                                      .CalcKind = CalcSettings.CalcKind,
                                                      .PolygonKind = CalcSettings.PolygonKind,
                                                      .StartPnt = startPnt,
                                                      .Transport = CalcSettings.Transport,
                                                      .AreaKind = userSettings.AreaKind,
                                                      .AreaRange = areaRange,
                                                      .PolygonLevel = CalcSettings.PolygonLevel,
                                                      .DonutPolygonLevel =
                                                        CalcSettings.DonutPolygonLevel,
                                                      .AreaDirection = userSettings.AreaDirection,
                                                      .PolygonDetail = CalcSettings.PolygonDetail}

            '--------------------------------------------------------------------------------------
            '  到達圏／流入圏計算の実行
            '--------------------------------------------------------------------------------------
            ' 計算結果を格納する変数を作成
            Dim pnts() As DCService.Pnt = Nothing
            Dim dcnwInfo As DCService.NWInfo = Nothing

            ' 到達圏／流入圏計算を実行
            Dim result As Integer = dcService.CalcArea3(dcEnviron, param, pnts, dcnwInfo)

            ' 最終距離計算日時を更新
            user_.UpdateLastDCTime()

            '--------------------------------------------------------------------------------------
            '  実行結果の取得
            '--------------------------------------------------------------------------------------
            If result = 0 Then
                ' 成功
                message.AppendLine("成功")
            Else
                ' 失敗、エラーメッセージを取得
                message.AppendLine("失敗")
                message.AppendLine(dcService.GetDCWSMessage(dcEnviron, result))
            End If

            '--------------------------------------------------------------------------------------
            '  ファイル出力
            '--------------------------------------------------------------------------------------
            If Not WritePnts(userSettings.OutputFilePath, pnts) Then
                ' 出力に失敗した場合
                message.AppendLine("ファイルの書き込みに失敗しました。")
            End If

        Catch ex As System.Net.WebException
            message.AppendLine("通信エラーが発生しました。")
            message.AppendLine(ex.Message)
        Catch
            '
        End Try

        Return message.ToString()
    End Function

    ''' <summary>
    ''' 計算を実行し実行結果をファイルに出力します。
    ''' </summary>
    ''' <param name="userSettings">ユーザー設定</param>
    ''' <returns>実行結果のメッセージ。</returns>
    Private Function DoWork(ByVal userSettings As UserSettings) As String
        Dim message As String = String.Empty
        ' 進捗フォームを表示して計算を実行
        ProgressDialog.Run(Sub()
                               message = DoCalc(userSettings)
                           End Sub, "計算中")

        Return message
    End Function

    ''' <summary>
    ''' 指定した正規表現パターンとNWIDが一致するデータを返します。
    ''' </summary>
    ''' <param name="nwInfos">対象のNWInfo配列</param>
    ''' <param name="pattern">NWIDの検索に使用する正規表現パターン</param>
    ''' <returns>NWIDが一致したNWInfo。一致するNWInfoがない場合はnull。</returns>
    Private Shared Function SelectNwInfo(ByVal nwInfos() As DCAdmin.NWInfo,
                                         ByVal pattern As String) As DCAdmin.NWInfo
        ' NWIDを正規表現で検索するクエリ
        Dim nwInfoQuery = From nwInfo In nwInfos
                          Where Regex.IsMatch(nwInfo.NWID.ToString(), pattern)
                          Order By nwInfo.NWID Descending
                          Select nwInfo

        ' クエリの先頭を返す
        Return nwInfoQuery.FirstOrDefault()
    End Function

    ''' <summary>
    ''' CSVファイルの全レコードを読み取り結果を返します。
    ''' </summary>
    ''' <param name="path">対象ファイルのパス</param>
    ''' <returns>読み取り結果のCsvRecord配列。読み取りに失敗した場合はnull。</returns>
    Private Shared Function ReadCsvFile(ByVal path As String) As CsvRecord()
        Dim records() As CsvRecord = Nothing
        Try
            Using strmReader As New StreamReader(path, Encoding.GetEncoding("shift_jis"))
                Using reader As New CsvReader(strmReader, False, Delimiter.Comma,
                                              QuotationMark.Double)
                    records = reader.ReadAllRecords()
                End Using
            End Using
        Catch
            '
        End Try

        Return records
    End Function

    ''' <summary>
    ''' ポイント構造体コレクションの経度と緯度をCSVファイルに書き込みます。
    ''' </summary>
    ''' <param name="path">出力ファイルのパス</param>
    ''' <param name="pnts">計算結果を格納したポイント構造体コレクション</param>
    ''' <returns>書き込みに成功した場合はtrue、失敗した場合はfalse。</returns>
    Private Shared Function WritePnts(ByVal path As String,
                                      ByVal pnts As IEnumerable(Of DCService.Pnt)) As Boolean
        Try
            Using strmWriter As New StreamWriter(path, False, Encoding.GetEncoding("shift_jis"))
                Using writer As New CsvWriter(strmWriter)
                    For Each pnt In pnts
                        If IsEmptyPnt(pnt) Then
                            ' 空白行がある場合は終了
                            Exit For
                        End If
                        Dim record As New CsvRecord(Delimiter.Comma, QuotationMark.Double)
                        record.Add(pnt.Lon)
                        record.Add(pnt.Lat)
                        writer.WriteRecord(record)
                    Next
                End Using
            End Using
        Catch
            Return False
        End Try

        Return True
    End Function

    ''' <summary>
    ''' ポイントが空かどうかを返します。
    ''' </summary>
    ''' <param name="pnt">対象のポイント</param>
    ''' <returns>空の場合はtrue、それ以外の場合はfalse。</returns>
    Private Shared Function IsEmptyPnt(ByVal pnt As DCService.Pnt) As Boolean
        Dim lon As Double = Double.Parse(pnt.Lon)
        Dim lat As Double = Double.Parse(pnt.Lat)

        Return lon = 0.0 And lat = 0.0
    End Function

    ''' <summary>
    ''' コントロールからユーザー設定を取得します。
    ''' </summary>
    ''' <returns>取得したUserSettingsオブジェクト。</returns>
    Private Function GetUserSettingsFromControls() As UserSettings
        Return New UserSettings() With {.UserID = textBoxUserId.Text,
                                        .Password = textBoxPassword.Text,
                                        .AdminUrl = textBoxAdminServerUrl.Text,
                                        .InputFilePath = textBoxInputFilePath.Text,
                                        .OutputFilePath = textBoxOutputFilePath.Text,
                                        .AreaDirection = If(radioButtonOutflow.Checked,
                                                            AreaDirection.OutFlow,
                                                            AreaDirection.InFlow),
                                        .AreaKind = If(radioButtonTime.Checked,
                                                       AreaKind.Time, AreaKind.Dist),
                                        .AreaRange = numericUpDownAreaRange.Value}
    End Function

    ''' <summary>
    ''' 指定したユーザー設定をコントロールに適用します。
    ''' </summary>
    ''' <param name="settings">適用するUserSettingsオブジェクト</param>
    Private Sub ApplyUserSettingsToControls(ByVal settings As UserSettings)
        textBoxUserId.Text = settings.UserID
        textBoxAdminServerUrl.Text = settings.AdminUrl
        textBoxInputFilePath.Text = settings.InputFilePath
        textBoxOutputFilePath.Text = settings.OutputFilePath
        radioButtonInflow.Checked = settings.AreaDirection = AreaDirection.InFlow
        radioButtonDist.Checked = settings.AreaKind = AreaKind.Dist
        numericUpDownAreaRange.Value = settings.AreaRange
    End Sub

    ''' <summary>
    ''' 入力ファイルを選択し、選択したファイルのパスを返します。
    ''' </summary>
    ''' <param name="oldPath">現在の入力ファイルパス</param>
    ''' <returns>選択したファイルのパス。キャンセルの場合はoldPath。</returns>
    Private Shared Function SelectInputFile(ByVal oldPath As String) As String
        Dim newPath As String = oldPath
        Using dialog As New OpenFileDialog()
            If File.Exists(oldPath) Then
                dialog.InitialDirectory = Path.GetDirectoryName(oldPath)
                dialog.FileName = Path.GetFileName(oldPath)
            End If
            dialog.Title = "入力ファイルの選択"
            dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*"

            If (dialog.ShowDialog() = DialogResult.OK) Then
                newPath = dialog.FileName
            End If
        End Using

        Return newPath
    End Function

    ''' <summary>
    ''' 出力ファイルを選択し、選択したファイルのパスを返します。
    ''' </summary>
    ''' <param name="oldPath">現在の出力ファイルパス</param>
    ''' <returns>選択したファイルのパス。キャンセルの場合はoldPath。</returns>
    Private Shared Function SelectOutputFile(ByVal oldPath As String) As String
        Dim newPath As String = oldPath
        Using dialog As New SaveFileDialog()
            If File.Exists(oldPath) Then
                dialog.InitialDirectory = Path.GetDirectoryName(oldPath)
                dialog.FileName = Path.GetFileName(oldPath)
            End If
            dialog.Title = "出力ファイルの選択"
            dialog.Filter = "CSVファイル (*.csv)|*.csv|全てのファイル (*.*)|*.*"

            If (dialog.ShowDialog() = DialogResult.OK) Then
                newPath = dialog.FileName
            End If
        End Using

        Return newPath
    End Function

    ''' <summary>
    ''' ユーザー設定を検証します。
    ''' </summary>
    ''' <param name="userID">ユーザーID</param>
    ''' <param name="password">パスワード</param>
    ''' <param name="adminUrl">管理サーバーURL</param>
    ''' <param name="inputFilePath">入力ファイルのパス</param>
    ''' <param name="outputFilePath">出力ファイルのパス</param>
    ''' <returns>設定が全て有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
    Private Shared Function ValidateSettings(ByVal userID As String, ByVal password As String,
                                             ByVal adminUrl As String,
                                             ByVal inputFilePath As String,
                                             ByVal outputFilePath As String) As String
        Dim message As New StringBuilder()
        If String.IsNullOrEmpty(userID) Then
            message.AppendLine("ユーザーIDが指定されていません。")
        End If
        If String.IsNullOrEmpty(password) Then
            message.AppendLine("パスワードが指定されていません。")
        End If
        If String.IsNullOrEmpty(adminUrl) Then
            message.AppendLine("管理サーバーURLが指定されていません。")
        End If
        Dim inputFilePathError = ValidateInputFilePath(inputFilePath)
        If inputFilePathError IsNot Nothing Then
            message.AppendLine(inputFilePathError)
        End If
        Dim outputFilePathError = ValidateOutputFilePath(outputFilePath)
        If outputFilePathError IsNot Nothing Then
            message.AppendLine(outputFilePathError)
        End If
        If message.Length = 0 Then
            If ConfirmOverwrite(outputFilePath) Then
                Return Nothing
            End If
            message.AppendLine("指定された " & outputFilePath & " は既に存在します。")
        End If

        Return message.ToString()
    End Function

    ''' <summary>
    ''' 入力ファイルのパスを検証します。
    ''' </summary>
    ''' <param name="value">検査対象のパス</param>
    ''' <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
    Private Shared Function ValidateInputFilePath(ByVal value As String) As String
        If String.IsNullOrEmpty(value) Then
            Return "入力ファイルが指定されていません。"
        End If
        Try
            If Not File.Exists(value) Then
                Return "指定された " & value & " が存在しません。"
            End If
        Catch
            Return "入力ファイルのパスが不正です。"
        End Try

        Return Nothing
    End Function

    ''' <summary>
    ''' 出力ファイルのパスを検証します。
    ''' </summary>
    ''' <param name="value">検査対象のパス</param>
    ''' <returns>有効な場合はnull、それ以外の場合はエラーメッセージ。</returns>
    Private Shared Function ValidateOutputFilePath(ByVal value As String) As String
        If String.IsNullOrEmpty(value) Then
            Return "出力ファイルが指定されていません。"
        End If
        Try
            Dim directoryName As String = Path.GetDirectoryName(value)
            If String.IsNullOrEmpty(directoryName) Then
                Return "指定されたフォルダは存在しません。"
            End If
            If Not Directory.Exists(directoryName) Then
                Return "指定された " & directoryName & " が存在しません。"
            End If
        Catch
            Return "出力ファイルのパスが不正です。"
        End Try

        Return Nothing
    End Function

    ''' <summary>
    ''' ファイルが既に存在する場合に上書き確認を行います。
    ''' </summary>
    ''' <param name="value">検査対象のパス</param>
    ''' <returns>ファイルが既に存在していて上書きしない場合はtrue、それ以外の場合はfalse。</returns>
    Private Shared Function ConfirmOverwrite(ByVal value As String) As Boolean
        If File.Exists(value) Then
            If MessageBox.Show(value & " は既に存在しています。" & Environment.NewLine &
                               "上書きしてよろしいですか？", "上書き確認",
                               MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation,
                               MessageBoxDefaultButton.Button2) = DialogResult.No Then
                Return False
            End If
        End If

        Return True
    End Function

    ''' <summary>
    ''' 再計算可能までの残待機時間を更新します。
    ''' </summary>
    ''' <param name="waitingSeconds">秒単位の残り待機時間</param>
    Private Sub UpdateWaitingTime(ByVal waitingSeconds As Long)
        Dim labelText As String = String.Empty
        If waitingSeconds > 0 Then
            ' 待機時間が経過していない場合
            labelText = String.Format("再計算可能まで {0:00} 秒", waitingSeconds)
        End If
        ' コントロールの更新
        labelCountDown.Text = labelText
        buttonCalc.Enabled = waitingSeconds <= 0
        timerCalcWait.Enabled = waitingSeconds > 0
    End Sub

#End Region

    '==============================================================================================
    '  Event Handlers
    '==============================================================================================
#Region "Event Handlers"

    ''' <summary>
    ''' メインフォームのロードイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' 保存したユーザー設定を復元し設定する
        userSettings_ = UserSettings.Load(UserSettings.SettingFilePath)
        If userSettings_ Is Nothing Then
            ' 失敗した場合は既定値を設定
            userSettings_ = New UserSettings()
            textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl
            Return
        End If
        ApplyUserSettingsToControls(userSettings_)
    End Sub

    ''' <summary>
    ''' メインフォームのクローズ後イベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainForm_FormClosed(sender As Object, e As FormClosedEventArgs) Handles MyBase.FormClosed
        ' ユーザー設定を保存する
        userSettings_ = GetUserSettingsFromControls()
        Try
            userSettings_.Save(UserSettings.SettingFilePath)
        Catch
            '
        End Try
    End Sub

    ''' <summary>
    ''' 「既定値」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonResetUrl_Click(sender As Object, e As EventArgs) Handles buttonResetUrl.Click
        textBoxAdminServerUrl.Text = CalcSettings.DefaultAdminUrl
    End Sub

    ''' <summary>
    ''' 「参照」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonSelectInputFile_Click(sender As Object, e As EventArgs) Handles buttonSelectInputFile.Click
        textBoxInputFilePath.Text = SelectInputFile(textBoxInputFilePath.Text)
    End Sub

    ''' <summary>
    ''' 「参照」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonSelectOutputFile_Click(sender As Object, e As EventArgs) Handles buttonSelectOutputFile.Click
        textBoxOutputFilePath.Text = SelectOutputFile(textBoxOutputFilePath.Text)
    End Sub

    ''' <summary>
    ''' 「時間（分）」ラジオボタンのチェック変更イベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub radioButtonTime_CheckedChanged(sender As Object, e As EventArgs) Handles radioButtonTime.CheckedChanged
        labelAreaKind.Text = If(radioButtonTime.Checked, "分", "km")
    End Sub

    ''' <summary>
    ''' 「計算範囲」のテキスト変更イベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub numericUpDownAreaRange_TextChanged(sender As Object, e As EventArgs) Handles numericUpDownAreaRange.TextChanged
        If numericUpDownAreaRange.Text = String.Empty Then
            numericUpDownAreaRange.Text = numericUpDownAreaRange.Value.ToString()
        End If
    End Sub

    ''' <summary>
    ''' 「計算」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonCalc_Click(sender As Object, e As EventArgs) Handles buttonCalc.Click
        ' フォームから値を取得し検証
        userSettings_ = GetUserSettingsFromControls()
        Dim errorMessage As String = ValidateSettings(userSettings_.UserID, userSettings_.Password,
                                                      userSettings_.AdminUrl,
                                                      userSettings_.InputFilePath,
                                                      userSettings_.OutputFilePath)
        If errorMessage IsNot Nothing Then
            ' 設定にエラーが存在する場合
            textBoxResults.Text = errorMessage
            Return
        End If

        ' テキストボックスの表示を変更
        textBoxResults.Text = "計算中..."

        ' 「計算」ボタンを無効
        buttonCalc.Enabled = False

        ' AdminURLを確認してSSL使用フラグを変更
        CalcSettings.UseSsl = userSettings_.AdminUrl.StartsWith("https")

        ' ユーザーを作成
        user_ = New User(userSettings_.UserID, userSettings_.Password, userSettings_.AdminUrl,
                         CalcSettings.UseSsl)

        ' 計算の実行
        textBoxResults.Text = DoWork(userSettings_)

        ' 再計算までのカウントダウン
        timerCalcWait.Start()
    End Sub

    ''' <summary>
    ''' 「終了」ボタンのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub buttonExit_Click(sender As Object, e As EventArgs) Handles buttonExit.Click
        Close()
    End Sub

    ''' <summary>
    ''' 「マニュアルを開く」リンクのクリックイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub linkLabelManual_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles linkLabelManual.LinkClicked
        Try
            Process.Start(UserSettings.UsersGuideFilePath)
        Catch

        End Try
    End Sub

    ''' <summary>
    ''' 計算待機時間タイマーのイベントです。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub timerCalcWait_Tick(sender As Object, e As EventArgs) Handles timerCalcWait.Tick
        If user_.UserInfo Is Nothing Then
            UpdateWaitingTime(-1)
            Return
        End If
        Dim elpsedSeconds As Long = CLng((DateTime.Now - user_.LastDCTime).TotalSeconds)
        Dim waitingSeconds As Long = CLng(user_.UserInfo.CalcInterval) - elpsedSeconds
        UpdateWaitingTime(waitingSeconds)
    End Sub

#End Region

End Class
